#line 124 "../mdcode.mdc"
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include "mdcode.h"

#line 894 "../mdcode.mdc"
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <string.h>
#line 867 "../mdcode.mdc"
static void copy_fname(char *name, int space, struct text t)
{
	char *sec = t.txt;
	int len = t.len;
	name[0] = 0;
	if (len < 5 || strncmp(sec, "File:", 5) != 0)
		return;
	sec += 5;
	len -= 5;
	while (len && sec[0] == ' ') {
		sec++;
		len--;
	}
	if (len >= space)
		len = space - 1;
	strncpy(name, sec, len);
	name[len] = 0;
}
#line 901 "../mdcode.mdc"
static int errs;
static void pr_err(char *msg)
{
	errs++;
	fprintf(stderr, "%s\n", msg);
}

static char *strnchr(char *haystack, int len, char needle)
{
	while (len > 0 && *haystack && *haystack != needle) {
		haystack++;
		len--;
	}
	return len > 0 && *haystack == needle ? haystack : NULL;
}

int main(int argc, char *argv[])
{
	int fd;
	size_t len;
	char *file;
	struct text section = {NULL, 0};
	struct section *table, *s, *prev;

	errs = 0;
	if (argc != 2 && argc != 3) {
		fprintf(stderr, "Usage: mdcode file.mdc [section]\n");
		exit(2);
	}
	if (argc == 3) {
		section.txt = argv[2];
		section.len = strlen(argv[2]);
	}

	fd = open(argv[1], O_RDONLY);
	if (fd < 0) {
		fprintf(stderr, "mdcode: cannot open %s: %s\n",
			argv[1], strerror(errno));
		exit(1);
	}
	len = lseek(fd, 0, 2);
	file = mmap(NULL, len, PROT_READ, MAP_SHARED, fd, 0);
	table = code_extract(file, file+len, pr_err);

	for (s = table; s;
		(code_free(s->code), prev = s, s = s->next, free(prev))) {
		FILE *fl;
		char fname[1024];
		char *spc = strnchr(s->section.txt, s->section.len, ' ');

		if (spc > s->section.txt && spc[-1] == ':') {
			if (strncmp(s->section.txt, "File: ", 6) != 0 &&
			    (section.txt == NULL ||
			     text_cmp(s->section, section) != 0))
				/* Ignore this section */
				continue;
		} else {
			fprintf(stderr, "Code in unreferenced section that is not ignored or a file name: %.*s\n",
				s->section.len, s->section.txt);
			errs++;
			continue;
		}
		if (section.txt) {
			if (text_cmp(s->section, section) == 0)
				code_node_print(stdout, s->code, argv[1]);
			break;
		}
		copy_fname(fname, sizeof(fname), s->section);
		if (fname[0] == 0) {
			fprintf(stderr, "Missing file name at:%.*s\n",
				s->section.len, s->section.txt);
			errs++;
			continue;
		}
		fl = fopen(fname, "w");
		if (!fl) {
			fprintf(stderr, "Cannot create %s: %s\n",
				fname, strerror(errno));
			errs++;
			continue;
		}
		code_node_print(fl, s->code, argv[1]);
		fclose(fl);
	}
	exit(!!errs);
}
#line 132 "../mdcode.mdc"

